@@ -1,7 +1,7 @@
 # scqubits: superconducting qubits in Python
 #
 # This file is part of scqubits: a Python package for superconducting qubits,
-# arXiv:2107.08552 (2021). https://arxiv.org/abs/2107.08552
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #     Copyright (c) 2019 and later, Jens Koch and Peter Groszkowski
 #     All rights reserved.
@@ -46,17 +46,25 @@ from scqubits.core.units import (
 )
 from scqubits.core.zeropi import ZeroPi
 from scqubits.core.zeropi_full import FullZeroPi
-from scqubits.explorer.explorer import Explorer
 
 # file IO
 from scqubits.io_utils.fileio import read, write
 
+
+# Import of custom-circuit modules needs to take place after other imports to
+# avoid circular import issues
+from scqubits.core.circuit import Circuit
+from scqubits.core.circuit_utils import truncation_template
+from scqubits.core.symbolic_circuit import SymbolicCircuit
+
+
 # GUI
 try:
+    from scqubits.ui.explorer_widget import Explorer
     from scqubits.ui.gui import GUI
 except NameError:
     warnings.warn(
-        "scqubits: could not import GUI - consider installing ipywidgets "
+        "scqubits: could not import GUI/Explorer - consider installing ipywidgets "
         "(optional dependency)?",
         ImportWarning,
     )
