@@ -1,6 +1,7 @@
 # qubit_base.py
 #
-# This file is part of scqubits.
+# This file is part of scqubits: a Python package for superconducting qubits,
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #    Copyright (c) 2019 and later, Jens Koch and Peter Groszkowski
 #    All rights reserved.
@@ -16,7 +17,17 @@ import functools
 import inspect
 
 from abc import ABC, ABCMeta, abstractmethod
-from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    Union,
+    overload,
+)
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -51,6 +62,15 @@ if IN_IPYTHON:
 else:
     from tqdm import tqdm
 
+if TYPE_CHECKING:
+    from typing_extensions import Literal
+
+    from scqubits.core.storage import WaveFunction
+
+
+LevelsTuple = Tuple[int, ...]
+Transition = Tuple[int, int]
+TransitionsTuple = Tuple[Transition, ...]
 
 # —Generic quantum system container and Qubit base class——————————————————————————————
 
@@ -58,10 +78,9 @@ else:
 class QuantumSystem(DispatchClient, ABC):
     """Generic quantum system class"""
 
-    truncated_dim = descriptors.WatchedProperty("QUANTUMSYSTEM_UPDATE")
+    truncated_dim = descriptors.WatchedProperty(int, "QUANTUMSYSTEM_UPDATE")
     _init_params: List[str]
     _image_filename: str
-    _evec_dtype: type
     _sys_type: str
 
     # To facilitate warnings in set_units, introduce a counter keeping track of the
@@ -71,7 +90,7 @@ class QuantumSystem(DispatchClient, ABC):
     # corresponding counts of instances
     _instance_counter: Dict[str, int] = {}
 
-    subclasses: List[ABCMeta] = []
+    _subclasses: List[ABCMeta] = []
 
     def __new__(cls, *args, **kwargs) -> "QuantumSystem":
         QuantumSystem._quantumsystem_counter += 1
@@ -98,11 +117,11 @@ class QuantumSystem(DispatchClient, ABC):
         self._id_str = id_str or self._autogenerate_id_str()
 
     def __init_subclass__(cls):
-        """Used to register all non-abstract subclasses as a list in
-        `QuantumSystem.subclasses`."""
+        """Used to register all non-abstract _subclasses as a list in
+        `QuantumSystem._subclasses`."""
         super().__init_subclass__()
         if not inspect.isabstract(cls):
-            cls.subclasses.append(cls)
+            cls._subclasses.append(cls)
 
     def __repr__(self) -> str:
         if hasattr(self, "_init_params"):
@@ -153,6 +172,26 @@ class QuantumSystem(DispatchClient, ABC):
     def hilbertdim(self) -> int:
         """Returns dimension of Hilbert space"""
 
+    @classmethod
+    def get_operator_names(cls) -> List[str]:
+        """Returns a list of all operator names for the quantum system.
+        Note that this list omits any operators that start with "_".
+
+        Parameters
+        ----------
+        subsys:
+            Class instance of quantum system
+
+        Returns
+        -------
+            list of operator names
+        """
+        operator_list = []
+        for name, val in inspect.getmembers(cls):
+            if "operator" in name and name[0] != "_" and name != "get_operator_names":
+                operator_list.append(name)
+        return operator_list
+
     @classmethod
     def create(cls) -> "QuantumSystem":
         """Use ipywidgets to create a new class instance"""
@@ -171,7 +210,7 @@ class QuantumSystem(DispatchClient, ABC):
 
     @staticmethod
     @abstractmethod
-    def default_params():
+    def default_params() -> Dict[str, Any]:
         """Return dictionary with default parameter values for initialization of
         class instance"""
 
@@ -199,9 +238,8 @@ class QubitBaseClass(QuantumSystem, ABC):
     """
 
     # see PEP 526 https://www.python.org/dev/peps/pep-0526/#class-and-instance-variable-annotations
-    truncated_dim: int
+    truncated_dim: int  # type:ignore
     _default_grid: Grid1d
-    _evec_dtype: type
     _sys_type: str
     _init_params: list
 
@@ -224,12 +262,30 @@ class QubitBaseClass(QuantumSystem, ABC):
         evals, evecs = order_eigensystem(evals, evecs)
         return evals, evecs
 
+    @overload
+    def eigenvals(
+        self,
+        evals_count: int = 6,
+        filename: str = None,
+        return_spectrumdata: "Literal[False]" = False,
+    ) -> ndarray:
+        ...
+
+    @overload
+    def eigenvals(
+        self,
+        evals_count: int,
+        filename: str,
+        return_spectrumdata: "Literal[True]",
+    ) -> SpectrumData:
+        ...
+
     def eigenvals(
         self,
         evals_count: int = 6,
         filename: str = None,
         return_spectrumdata: bool = False,
-    ) -> ndarray:
+    ) -> Union[SpectrumData, ndarray]:
         """Calculates eigenvalues using `scipy.linalg.eigh`, returns numpy array of
         eigenvalues.
 
@@ -257,12 +313,30 @@ class QubitBaseClass(QuantumSystem, ABC):
             specdata.filewrite(filename)
         return specdata if return_spectrumdata else evals
 
+    @overload
+    def eigensys(
+        self,
+        evals_count: int = 6,
+        filename: str = None,
+        return_spectrumdata: "Literal[False]" = False,
+    ) -> Tuple[ndarray, ndarray]:
+        ...
+
+    @overload
+    def eigensys(
+        self,
+        evals_count: int,
+        filename: Optional[str],
+        return_spectrumdata: "Literal[True]",
+    ) -> SpectrumData:
+        ...
+
     def eigensys(
         self,
         evals_count: int = 6,
         filename: str = None,
         return_spectrumdata: bool = False,
-    ) -> Tuple[ndarray, ndarray]:
+    ) -> Union[Tuple[ndarray, ndarray], SpectrumData]:
         """Calculates eigenvalues and corresponding eigenvectors using
         `scipy.linalg.eigh`. Returns two numpy arrays containing the eigenvalues and
         eigenvectors, respectively.
@@ -289,7 +363,29 @@ class QubitBaseClass(QuantumSystem, ABC):
             )
         if filename:
             specdata.filewrite(filename)
-        return specdata if return_spectrumdata else (evals, evecs)  # type: ignore
+        return specdata if return_spectrumdata else (evals, evecs)
+
+    @overload
+    def matrixelement_table(
+        self,
+        operator: str,
+        evecs: ndarray = None,
+        evals_count: int = 6,
+        filename: str = None,
+        return_datastore: "Literal[False]" = False,
+    ) -> ndarray:
+        ...
+
+    @overload
+    def matrixelement_table(
+        self,
+        operator: str,
+        evecs: ndarray,
+        evals_count: int,
+        filename: Optional[str],
+        return_datastore: "Literal[True]",
+    ) -> DataStore:
+        ...
 
     def matrixelement_table(
         self,
@@ -298,7 +394,7 @@ class QubitBaseClass(QuantumSystem, ABC):
         evals_count: int = 6,
         filename: str = None,
         return_datastore: bool = False,
-    ) -> ndarray:
+    ) -> Union[DataStore, ndarray]:
         """Returns table of matrix elements for `operator` with respect to the
         eigenstates of the qubit. The operator is given as a string matching a class
         method returning an operator matrix. E.g., for an instance `trm` of Transmon,
@@ -336,9 +432,9 @@ class QubitBaseClass(QuantumSystem, ABC):
 
     def _esys_for_paramval(
         self, paramval: float, param_name: str, evals_count: int
-    ) -> Union[Tuple[ndarray, ndarray], SpectrumData]:
+    ) -> Tuple[ndarray, ndarray]:
         setattr(self, param_name, paramval)
-        return self.eigensys(evals_count)
+        return self.eigensys(evals_count=evals_count)
 
     def _evals_for_paramval(
         self, paramval: float, param_name: str, evals_count: int
@@ -387,28 +483,29 @@ class QubitBaseClass(QuantumSystem, ABC):
 
         target_map = get_map_method(num_cpus)
         if not get_eigenstates:
-            func = functools.partial(
+            func_evals = functools.partial(
                 self._evals_for_paramval, param_name=param_name, evals_count=evals_count
             )
             with InfoBar(
                 "Parallel computation of eigensystems [num_cpus={}]".format(num_cpus),
                 num_cpus,
             ):
-                eigenvalue_table = list(
-                    target_map(
-                        func,
-                        tqdm(
-                            param_vals,
-                            desc="Spectral data",
-                            leave=False,
-                            disable=tqdm_disable,
-                        ),
+                eigenvalue_table = np.asarray(
+                    list(
+                        target_map(
+                            func_evals,
+                            tqdm(
+                                param_vals,
+                                desc="Spectral data",
+                                leave=False,
+                                disable=tqdm_disable,
+                            ),
+                        )
                     )
                 )
-            eigenvalue_table = np.asarray(eigenvalue_table)
             eigenstate_table = None
         else:
-            func = functools.partial(
+            func_esys = functools.partial(
                 self._esys_for_paramval, param_name=param_name, evals_count=evals_count
             )
             with InfoBar(
@@ -420,7 +517,7 @@ class QubitBaseClass(QuantumSystem, ABC):
                 # dimension can vary with paramvals
                 eigensystem_mapdata = list(
                     target_map(
-                        func,
+                        func_esys,
                         tqdm(
                             param_vals,
                             desc="Spectral data",
@@ -447,22 +544,15 @@ class QubitBaseClass(QuantumSystem, ABC):
         )
         if filename:
             specdata.filewrite(filename)
-
-        return SpectrumData(
-            eigenvalue_table,
-            self.get_initdata(),
-            param_name,
-            param_vals,
-            state_table=eigenstate_table,
-        )
+        return specdata
 
     def _compute_dispersion(
         self,
         dispersion_name: str,
         param_name: str,
         param_vals: ndarray,
-        transitions: Union[Tuple[int], Tuple[Tuple[int], ...]] = (0, 1),
-        levels: Optional[Union[int, Tuple[int]]] = None,
+        transitions_tuple: TransitionsTuple = ((0, 1),),
+        levels_tuple: Optional[LevelsTuple] = None,
         point_count: int = 50,
         num_cpus: Optional[int] = None,
     ) -> Tuple[ndarray, ndarray]:
@@ -481,7 +571,9 @@ class QubitBaseClass(QuantumSystem, ABC):
 
         previous_dispval = getattr(self, dispersion_name)
         previous_paramval = getattr(self, param_name)
-        max_level = np.max(transitions) if not levels else np.max(levels)
+        max_level = (
+            np.max(transitions_tuple) if not levels_tuple else np.max(levels_tuple)
+        )
         sweep = ParameterSweep(
             hilbertspace,
             paramvals_by_name,
@@ -490,18 +582,18 @@ class QubitBaseClass(QuantumSystem, ABC):
             bare_only=True,
             num_cpus=num_cpus,
         )
-        eigenenergies = sweep["bare_evals"]["subsys":0].toarray()
+        eigenenergies = sweep["bare_evals"]["subsys":0].toarray()  # type:ignore
 
-        if levels is None:
-            dispersions = np.empty((len(transitions), len(param_vals)))
-            for index, (i, j) in enumerate(transitions):
+        if levels_tuple is None:
+            dispersions = np.empty((len(transitions_tuple), len(param_vals)))
+            for index, (i, j) in enumerate(transitions_tuple):
                 energy_ij = eigenenergies[:, :, i] - eigenenergies[:, :, j]
                 dispersions[index] = np.max(energy_ij, axis=0) - np.min(
                     energy_ij, axis=0
                 )
         else:
-            dispersions = np.empty((len(levels), len(param_vals)))
-            for index, j in enumerate(levels):
+            dispersions = np.empty((len(levels_tuple), len(param_vals)))
+            for index, j in enumerate(levels_tuple):
                 energy_j = eigenenergies[:, :, j]
                 dispersions[index] = np.max(energy_j, axis=0) - np.min(energy_j, axis=0)
 
@@ -515,8 +607,8 @@ class QubitBaseClass(QuantumSystem, ABC):
         param_name: str,
         param_vals: ndarray,
         ref_param: Optional[str] = None,
-        transitions: Union[Tuple[int], Tuple[Tuple[int], ...]] = (0, 1),
-        levels: Optional[Union[int, Tuple[int]]] = None,
+        transitions: Union[Transition, TransitionsTuple] = (0, 1),
+        levels: Optional[Union[int, LevelsTuple]] = None,
         point_count: int = 50,
         num_cpus: Optional[int] = None,
     ) -> SpectrumData:
@@ -552,18 +644,42 @@ class QubitBaseClass(QuantumSystem, ABC):
             number of cores to be used for computation
             (default value: settings.NUM_CPUS)
         """
-
-        if isinstance(levels, int):
-            levels = (levels,)
+        if levels is not None:
+            if isinstance(levels, int):
+                # presence of levels argument will overwrite `transitions`;
+                # here: single level
+                levels_tuple: Optional[LevelsTuple] = (levels,)
+                transitions_tuple: TransitionsTuple = (transitions,)  # type:ignore
+            elif isinstance(levels, tuple):
+                # presence of levels argument will overwrite `transitions`;
+                # here: multiple levels
+                levels_tuple: Optional[LevelsTuple] = levels
+                transitions_tuple: TransitionsTuple = (transitions,)  # type:ignore
+            else:
+                raise ValueError(
+                    "Invalid `levels` specification: expect int or tuple " "of int"
+                )
         elif isinstance(transitions[0], int):
-            transitions = (transitions,)
+            # transitions is inferred to be of form (i, j), so only a single one
+            transitions_tuple = (transitions,)  # type:ignore
+            levels_tuple = None
+        elif isinstance(transitions[0], tuple):
+            # transitions is inferred to be of form ((i1, j1), ...) ,
+            # there are multiple transitions
+            transitions_tuple = transitions
+            levels_tuple = None
+        else:
+            raise ValueError(
+                "Invalid `transitions` specification: expect either ("
+                "int, int)  or ((int, int), ...)"
+            )
 
         eigenenergies, dispersion = self._compute_dispersion(
             dispersion_name,
             param_name,
             param_vals,
-            transitions=transitions,
-            levels=levels,
+            transitions_tuple=transitions_tuple,
+            levels_tuple=levels_tuple,
             point_count=point_count,
             num_cpus=num_cpus,
         )
@@ -577,7 +693,7 @@ class QubitBaseClass(QuantumSystem, ABC):
             self.get_initdata(),
             param_name,
             param_vals,
-            labels=levels or transitions,
+            labels=levels_tuple or transitions_tuple,
             dispersion=dispersion.T,
         )
         return specdata
@@ -622,13 +738,9 @@ class QubitBaseClass(QuantumSystem, ABC):
             shape=(paramvals_count, evals_count, evals_count), dtype=np.complex_
         )
 
-        for index, paramval in tqdm(
-            enumerate(param_vals),
-            total=len(param_vals),
-            disable=settings.PROGRESSBAR_DISABLED,
-            leave=False,
-        ):
-            evecs = spectrumdata.state_table[index]  # type: ignore
+        assert spectrumdata.state_table is not None
+        for index, paramval in enumerate(param_vals):
+            evecs = spectrumdata.state_table[index]
             matelem_table[index] = self.matrixelement_table(
                 operator, evecs=evecs, evals_count=evals_count
             )
@@ -683,8 +795,8 @@ class QubitBaseClass(QuantumSystem, ABC):
         param_name: str,
         param_vals: ndarray,
         ref_param: Optional[str] = None,
-        transitions: Union[Tuple[int], Tuple[Tuple[int], ...]] = (0, 1),
-        levels: Optional[Union[int, Tuple[int]]] = None,
+        transitions: Union[Transition, TransitionsTuple] = (0, 1),
+        levels: Optional[Union[int, LevelsTuple]] = None,
         point_count: int = 50,
         num_cpus: Optional[int] = None,
         **kwargs,
@@ -732,17 +844,19 @@ class QubitBaseClass(QuantumSystem, ABC):
             num_cpus=num_cpus,
         )
         if levels is not None:
-            if isinstance(levels, int):
-                levels = (levels,)
-            label_list = [str(j) for j in levels]
+            levels_tuple = levels if isinstance(levels, tuple) else (levels,)
+            label_list = [str(j) for j in levels_tuple]
         else:
-            if isinstance(transitions[0], int):
-                transitions = (transitions,)
-            label_list = ["{}{}".format(i, j) for i, j in transitions]
+            transitions_tuple: TransitionsTuple = (
+                transitions  # type:ignore
+                if isinstance(transitions[0], tuple)
+                else (transitions,)
+            )
+            label_list = ["{}{}".format(i, j) for i, j in transitions_tuple]
 
         return plot.data_vs_paramvals(
-            xdata=specdata.param_vals,
-            ydata=specdata.dispersion,
+            xdata=specdata.param_vals,  # type:ignore
+            ydata=specdata.dispersion,  # type:ignore
             label_list=label_list,
             xlabel=specdata.param_name,
             ylabel="energy dispersion [{}]".format(units.get_units()),
@@ -788,12 +902,19 @@ class QubitBaseClass(QuantumSystem, ABC):
             standard plotting option (see separate documentation)
         """
         matrixelem_array = self.matrixelement_table(operator, evecs, evals_count)
+        assert isinstance(matrixelem_array, np.ndarray)
         if not show3d:
             return plot.matrix2d(
-                matrixelem_array, mode=mode, show_numbers=show_numbers, **kwargs
+                matrixelem_array,
+                mode=mode,
+                show_numbers=show_numbers,
+                **kwargs,
             )
         return plot.matrix(
-            matrixelem_array, mode=mode, show_numbers=show_numbers, **kwargs
+            matrixelem_array,
+            mode=mode,
+            show_numbers=show_numbers,
+            **kwargs,
         )
 
     def plot_matelem_vs_paramvals(
@@ -883,14 +1004,18 @@ class QubitBaseClass1d(QubitBaseClass):
 
     # see PEP 526 https://www.python.org/dev/peps/pep-0526/#class-and-instance-variable-annotations
     _default_grid: Grid1d
-    _evec_dtype = np.float_
 
     @abstractmethod
     def potential(self, phi: Union[float, ndarray]) -> Union[float, ndarray]:
         pass
 
     @abstractmethod
-    def wavefunction(self, esys: ndarray, which: int = 0, phi_grid: Grid1d = None):
+    def wavefunction(
+        self,
+        esys: Optional[Tuple[ndarray, ndarray]],
+        which: int = 0,
+        phi_grid: Grid1d = None,
+    ) -> "WaveFunction":
         pass
 
     def wavefunction1d_defaults(
@@ -947,7 +1072,8 @@ class QubitBaseClass1d(QubitBaseClass):
 
         if esys is None:
             evals_count = max(wavefunc_indices) + 1
-            evals = self.eigenvals(evals_count=evals_count)
+            esys = self.eigensys(evals_count=evals_count)
+            evals, _ = esys
         else:
             evals, _ = esys
 
@@ -970,7 +1096,7 @@ class QubitBaseClass1d(QubitBaseClass):
         kwargs["fig_ax"] = fig_ax
         kwargs = {
             **self.wavefunction1d_defaults(
-                mode, evals, wavefunc_count=len(wavefunc_indices)
+                mode, evals, wavefunc_count=len(wavefunc_indices)  # type:ignore
             ),
             **kwargs,
         }
@@ -979,7 +1105,7 @@ class QubitBaseClass1d(QubitBaseClass):
 
         plot.wavefunction1d(
             wavefunctions,
-            potential_vals=potential_vals,
+            potential_vals=potential_vals,  # type:ignore
             offset=energies,
             scaling=scaling,
             **kwargs,
