@@ -1,6 +1,7 @@
 # qubit_widget.py
 #
-# This file is part of scqubits.
+# This file is part of scqubits: a Python package for superconducting qubits,
+# Quantum 5, 583 (2021). https://quantum-journal.org/papers/q-2021-11-17-583/
 #
 #    Copyright (c) 2019 and later, Jens Koch and Peter Groszkowski
 #    All rights reserved.
@@ -52,7 +53,7 @@ def create_widget(
         label_str = name
         # NOTE: This will break if names of energy parameters in future qubits do not start with 'E'
         if name[0] == "E":
-            label_str += " [" + units.get_units() + "]"
+            label_str += f" [{units.get_units()}]"
         elif name == "flux":
             label_str += r" [$\Phi_0$]"
         label = ipywidgets.Label(value=label_str)
